/// <reference types="jquery" />
/// <reference types="bootstrap" />
declare module "util" {
    export function lpad(s: string, n: number): string;
    export function rpad(s: string, n: number): string;
    export function byte2signed(b: number): number;
    export function getFilenameForPath(s: string): string;
    export function getFolderForPath(s: string): string;
    export function getFilenamePrefix(s: string): string;
    export function hex(v: number, nd?: number): string;
    export function tobin(v: number, nd?: number): string;
    export function toradix(v: number, nd: number, radix: number): string;
    export function arrayCompare(a: any[], b: any[]): boolean;
    export function invertMap(m: {}): {};
    export function highlightDifferences(s1: string, s2: string): string;
    export function lzgmini(): void;
    export function stringToByteArray(s: string): Uint8Array;
    export function byteArrayToString(data: number[] | Uint8Array): string;
    export function byteArrayToUTF8(data: number[] | Uint8Array): string;
    export function removeBOM(s: string): string;
    export function isProbablyBinary(path: string, data?: number[] | Uint8Array): boolean;
    export function compressLZG(em_module: any, inBuffer: number[], levelArg?: boolean): Uint8Array;
    export function safe_extend(deep: any, dest: any, src: any): any;
    export function printFlags(val: number, names: string[], r2l: boolean): string;
    export function rgb2bgr(x: any): number;
    export function RGBA(r: number, g: number, b: number): number;
    export function clamp(minv: number, maxv: number, v: number): number;
    export function safeident(s: string): string;
    export function rle_unpack(src: Uint8Array): Uint8Array;
    export function getWithBinary(url: string, success: (text: string | Uint8Array) => void, datatype: 'text' | 'arraybuffer'): void;
    export function getBasePlatform(platform: string): string;
    export function getRootPlatform(platform: string): string;
    export function getRootBasePlatform(platform: string): string;
}
declare module "emu" {
    export var PLATFORMS: {};
    export function noise(): number;
    export function getNoiseSeed(): number;
    export function setNoiseSeed(x: number): void;
    type KeyboardCallback = (which: number, charCode: number, flags: KeyFlags) => void;
    export enum KeyFlags {
        KeyDown = 1,
        Shift = 2,
        Ctrl = 4,
        Alt = 8,
        Meta = 16,
        KeyUp = 64,
        KeyPress = 128
    }
    export function _setKeyboardEvents(canvas: HTMLElement, callback: KeyboardCallback): void;
    type VideoCanvasOptions = {
        rotate?: number;
        overscan?: boolean;
    };
    export class RasterVideo {
        mainElement: HTMLElement;
        width: number;
        height: number;
        options: VideoCanvasOptions;
        constructor(mainElement: HTMLElement, width: number, height: number, options?: VideoCanvasOptions);
        canvas: HTMLCanvasElement;
        ctx: any;
        imageData: any;
        arraybuf: any;
        buf8: any;
        datau32: any;
        vcanvas: JQuery;
        paddle_x: number;
        paddle_y: number;
        setRotate(rotate: number): void;
        create(): void;
        setKeyboardEvents(callback: any): void;
        getFrameData(): any;
        getContext(): any;
        updateFrame(sx?: number, sy?: number, dx?: number, dy?: number, w?: number, h?: number): void;
        clearRect(dx: number, dy: number, w: number, h: number): void;
        setupMouseEvents(el?: HTMLCanvasElement): void;
    }
    export class VectorVideo extends RasterVideo {
        persistenceAlpha: number;
        jitter: number;
        gamma: number;
        sx: number;
        sy: number;
        create(): void;
        clear(): void;
        COLORS: string[];
        drawLine(x1: number, y1: number, x2: number, y2: number, intensity: number, color: number): void;
    }
    export class RAM {
        mem: Uint8Array;
        constructor(size: number);
    }
    export class EmuHalt extends Error {
    }
    export class AnimationTimer {
        callback: any;
        running: boolean;
        pulsing: boolean;
        lastts: number;
        useReqAnimFrame: boolean;
        nframes: any;
        startts: any;
        frameRate: any;
        intervalMsec: any;
        constructor(frequencyHz: number, callback: () => void);
        scheduleFrame(msec: number): void;
        nextFrame(ts?: number): void;
        isRunning(): boolean;
        start(): void;
        stop(): void;
    }
    export function dumpRAM(ram: Uint8Array | number[], ramofs: number, ramlen: number): string;
    export interface KeyDef {
        c: number;
        n: string;
        plyr?: number;
        xaxis?: number;
        yaxis?: number;
        button?: number;
    }
    export interface KeyMapEntry {
        index: number;
        mask: number;
        def: KeyDef;
    }
    type KeyCodeMap = Map<number, KeyMapEntry>;
    export const Keys: {
        ANYKEY: {
            c: number;
            n: string;
        };
        UP: {
            c: number;
            n: string;
            plyr: number;
            yaxis: number;
        };
        DOWN: {
            c: number;
            n: string;
            plyr: number;
            yaxis: number;
        };
        LEFT: {
            c: number;
            n: string;
            plyr: number;
            xaxis: number;
        };
        RIGHT: {
            c: number;
            n: string;
            plyr: number;
            xaxis: number;
        };
        A: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        B: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        GP_A: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        GP_B: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        SELECT: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        START: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        P2_UP: {
            c: number;
            n: string;
            plyr: number;
            yaxis: number;
        };
        P2_DOWN: {
            c: number;
            n: string;
            plyr: number;
            yaxis: number;
        };
        P2_LEFT: {
            c: number;
            n: string;
            plyr: number;
            xaxis: number;
        };
        P2_RIGHT: {
            c: number;
            n: string;
            plyr: number;
            xaxis: number;
        };
        P2_A: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        P2_B: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        P2_SELECT: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        P2_START: {
            c: number;
            n: string;
            plyr: number;
            button: number;
        };
        VK_ESCAPE: {
            c: number;
            n: string;
        };
        VK_F1: {
            c: number;
            n: string;
        };
        VK_F2: {
            c: number;
            n: string;
        };
        VK_F3: {
            c: number;
            n: string;
        };
        VK_F4: {
            c: number;
            n: string;
        };
        VK_F5: {
            c: number;
            n: string;
        };
        VK_F6: {
            c: number;
            n: string;
        };
        VK_F7: {
            c: number;
            n: string;
        };
        VK_F8: {
            c: number;
            n: string;
        };
        VK_F9: {
            c: number;
            n: string;
        };
        VK_F10: {
            c: number;
            n: string;
        };
        VK_F11: {
            c: number;
            n: string;
        };
        VK_F12: {
            c: number;
            n: string;
        };
        VK_SCROLL_LOCK: {
            c: number;
            n: string;
        };
        VK_PAUSE: {
            c: number;
            n: string;
        };
        VK_QUOTE: {
            c: number;
            n: string;
        };
        VK_1: {
            c: number;
            n: string;
        };
        VK_2: {
            c: number;
            n: string;
        };
        VK_3: {
            c: number;
            n: string;
        };
        VK_4: {
            c: number;
            n: string;
        };
        VK_5: {
            c: number;
            n: string;
        };
        VK_6: {
            c: number;
            n: string;
        };
        VK_7: {
            c: number;
            n: string;
        };
        VK_8: {
            c: number;
            n: string;
        };
        VK_9: {
            c: number;
            n: string;
        };
        VK_0: {
            c: number;
            n: string;
        };
        VK_MINUS: {
            c: number;
            n: string;
        };
        VK_MINUS2: {
            c: number;
            n: string;
        };
        VK_EQUALS: {
            c: number;
            n: string;
        };
        VK_EQUALS2: {
            c: number;
            n: string;
        };
        VK_BACK_SPACE: {
            c: number;
            n: string;
        };
        VK_TAB: {
            c: number;
            n: string;
        };
        VK_Q: {
            c: number;
            n: string;
        };
        VK_W: {
            c: number;
            n: string;
        };
        VK_E: {
            c: number;
            n: string;
        };
        VK_R: {
            c: number;
            n: string;
        };
        VK_T: {
            c: number;
            n: string;
        };
        VK_Y: {
            c: number;
            n: string;
        };
        VK_U: {
            c: number;
            n: string;
        };
        VK_I: {
            c: number;
            n: string;
        };
        VK_O: {
            c: number;
            n: string;
        };
        VK_P: {
            c: number;
            n: string;
        };
        VK_ACUTE: {
            c: number;
            n: string;
        };
        VK_OPEN_BRACKET: {
            c: number;
            n: string;
        };
        VK_CLOSE_BRACKET: {
            c: number;
            n: string;
        };
        VK_CAPS_LOCK: {
            c: number;
            n: string;
        };
        VK_A: {
            c: number;
            n: string;
        };
        VK_S: {
            c: number;
            n: string;
        };
        VK_D: {
            c: number;
            n: string;
        };
        VK_F: {
            c: number;
            n: string;
        };
        VK_G: {
            c: number;
            n: string;
        };
        VK_H: {
            c: number;
            n: string;
        };
        VK_J: {
            c: number;
            n: string;
        };
        VK_K: {
            c: number;
            n: string;
        };
        VK_L: {
            c: number;
            n: string;
        };
        VK_CEDILLA: {
            c: number;
            n: string;
        };
        VK_TILDE: {
            c: number;
            n: string;
        };
        VK_ENTER: {
            c: number;
            n: string;
        };
        VK_SHIFT: {
            c: number;
            n: string;
        };
        VK_BACK_SLASH: {
            c: number;
            n: string;
        };
        VK_Z: {
            c: number;
            n: string;
        };
        VK_X: {
            c: number;
            n: string;
        };
        VK_C: {
            c: number;
            n: string;
        };
        VK_V: {
            c: number;
            n: string;
        };
        VK_B: {
            c: number;
            n: string;
        };
        VK_N: {
            c: number;
            n: string;
        };
        VK_M: {
            c: number;
            n: string;
        };
        VK_COMMA: {
            c: number;
            n: string;
        };
        VK_PERIOD: {
            c: number;
            n: string;
        };
        VK_SEMICOLON: {
            c: number;
            n: string;
        };
        VK_SLASH: {
            c: number;
            n: string;
        };
        VK_CONTROL: {
            c: number;
            n: string;
        };
        VK_ALT: {
            c: number;
            n: string;
        };
        VK_SPACE: {
            c: number;
            n: string;
        };
        VK_INSERT: {
            c: number;
            n: string;
        };
        VK_DELETE: {
            c: number;
            n: string;
        };
        VK_HOME: {
            c: number;
            n: string;
        };
        VK_END: {
            c: number;
            n: string;
        };
        VK_PAGE_UP: {
            c: number;
            n: string;
        };
        VK_PAGE_DOWN: {
            c: number;
            n: string;
        };
        VK_UP: {
            c: number;
            n: string;
        };
        VK_DOWN: {
            c: number;
            n: string;
        };
        VK_LEFT: {
            c: number;
            n: string;
        };
        VK_RIGHT: {
            c: number;
            n: string;
        };
        VK_NUM_LOCK: {
            c: number;
            n: string;
        };
        VK_DIVIDE: {
            c: number;
            n: string;
        };
        VK_MULTIPLY: {
            c: number;
            n: string;
        };
        VK_SUBTRACT: {
            c: number;
            n: string;
        };
        VK_ADD: {
            c: number;
            n: string;
        };
        VK_DECIMAL: {
            c: number;
            n: string;
        };
        VK_NUMPAD0: {
            c: number;
            n: string;
        };
        VK_NUMPAD1: {
            c: number;
            n: string;
        };
        VK_NUMPAD2: {
            c: number;
            n: string;
        };
        VK_NUMPAD3: {
            c: number;
            n: string;
        };
        VK_NUMPAD4: {
            c: number;
            n: string;
        };
        VK_NUMPAD5: {
            c: number;
            n: string;
        };
        VK_NUMPAD6: {
            c: number;
            n: string;
        };
        VK_NUMPAD7: {
            c: number;
            n: string;
        };
        VK_NUMPAD8: {
            c: number;
            n: string;
        };
        VK_NUMPAD9: {
            c: number;
            n: string;
        };
        VK_NUMPAD_CENTER: {
            c: number;
            n: string;
        };
    };
    type KeyMapFunction = (o: KeyMapEntry, key: number, code: number, flags: number) => void;
    export function newKeyboardHandler(switches: number[] | Uint8Array, map: KeyCodeMap, func?: KeyMapFunction, alwaysfunc?: boolean): (key: number, code: number, flags: number) => void;
    export function setKeyboardFromMap(video: RasterVideo, switches: number[] | Uint8Array, map: KeyCodeMap, func?: KeyMapFunction, alwaysfunc?: boolean): ControllerPoller;
    export function makeKeycodeMap(table: [KeyDef, number, number][]): KeyCodeMap;
    export class ControllerPoller {
        active: boolean;
        handler: any;
        state: Int8Array;
        lastState: Int8Array;
        AXIS0: number;
        constructor(handler: (key: any, code: any, flags: any) => void);
        poll(): void;
        handleStateChange(gpi: number, k: number): void;
    }
    export function padBytes(data: Uint8Array | number[], len: number, padstart?: boolean): Uint8Array;
    type AddressReadWriteFn = ((a: number) => number) | ((a: number, v: number) => void);
    type AddressDecoderEntry = [number, number, number, AddressReadWriteFn];
    type AddressDecoderOptions = {
        gmask?: number;
    };
    export function AddressDecoder(table: AddressDecoderEntry[], options?: AddressDecoderOptions): any;
    export function newAddressDecoder(table: AddressDecoderEntry[], options?: AddressDecoderOptions): (a: number, v?: number) => number;
    export class Toolbar {
        span: JQuery;
        grp: JQuery;
        mousetrap: any;
        boundkeys: any[];
        constructor(parentDiv: HTMLElement, focusDiv: HTMLElement);
        destroy(): void;
        newGroup(): JQuery<HTMLSpanElement>;
        add(key: string, alttext: string, icon: string, fn: (e: any, combo: any) => void): any;
    }
    export function getMousePos(canvas: HTMLCanvasElement, evt: any): {
        x: number;
        y: number;
    };
}
declare module "workertypes" {
    export type FileData = string | Uint8Array;
    export interface SourceLine {
        offset: number;
        line: number;
        insns?: string;
        iscode?: boolean;
        cycles?: number;
    }
    export class SourceFile {
        lines: SourceLine[];
        text: string;
        offset2line: Map<number, number>;
        line2offset: Map<number, number>;
        constructor(lines: SourceLine[], text: string);
        findLineForOffset(PC: number, lookbehind: number): {
            line: any;
            offset: number;
        };
        lineCount(): number;
    }
    export interface Dependency {
        path: string;
        filename: string;
        link: boolean;
        data: FileData;
    }
    export interface WorkerFileUpdate {
        path: string;
        data: FileData;
    }
    export interface WorkerBuildStep {
        path?: string;
        platform: string;
        tool: string;
        mainfile?: boolean;
    }
    export interface WorkerMessage extends WorkerBuildStep {
        preload: string;
        reset: boolean;
        code: string;
        updates: WorkerFileUpdate[];
        buildsteps: WorkerBuildStep[];
    }
    export interface WorkerError {
        line: number;
        msg: string;
        path?: string;
    }
    export interface CodeListing {
        lines: SourceLine[];
        asmlines?: SourceLine[];
        text?: string;
        sourcefile?: SourceFile;
        assemblyfile?: SourceFile;
    }
    export type CodeListingMap = {
        [path: string]: CodeListing;
    };
    export type VerilogOutput = {
        program_rom_variable: string;
        program_rom: Uint8Array;
        code: string;
        name: string;
        ports: any[];
        signals: any[];
    };
    export type WorkerOutput = Uint8Array | VerilogOutput;
    export type Segment = {
        name: string;
        start: number;
        size: number;
        last?: number;
        type?: string;
    };
    export interface WorkerResult {
        output: WorkerOutput;
        errors: WorkerError[];
        listings: CodeListingMap;
        symbolmap: {
            [sym: string]: number;
        };
        params: {};
        segments?: Segment[];
        unchanged?: boolean;
    }
}
declare module "cpu/disasm6502" {
    export var OPS_6502: {
        mn: string;
        am: string;
        nb: number;
        il: number;
        c1: number;
        c2: number;
    }[];
    export function disassemble6502(pc: number, b0: number, b1: number, b2: number): {
        line: string;
        nbytes: number;
        isaddr: boolean;
    };
}
declare module "cpu/disasmz80" {
    export function disassembleZ80(pc: number, b0: number, b1: number, b2: number, b3: number): {
        line: string;
        nbytes: number;
        isaddr: boolean;
    };
}
declare module "devices" {
    export interface SavesState<S> {
        saveState(): S;
        loadState(state: S): void;
    }
    export interface Bus {
        read(a: number): number;
        write(a: number, v: number): void;
        readConst?(a: number): number;
    }
    export interface ClockBased {
        advanceClock(): void;
    }
    export interface InstructionBased {
        advanceInsn(): number;
    }
    export type TrapCondition = () => boolean;
    export interface FrameBased {
        advanceFrame(trap: TrapCondition): number;
    }
    export interface VideoSource {
        getVideoParams(): VideoParams;
        connectVideo(pixels: Uint32Array): void;
    }
    export interface RasterFrameBased extends FrameBased, VideoSource {
        getRasterY(): number;
        getRasterX(): number;
    }
    export interface VideoParams {
        width: number;
        height: number;
        overscan?: boolean;
        rotate?: number;
        videoFrequency?: number;
    }
    export interface SampledAudioParams {
        sampleRate: number;
        stereo: boolean;
    }
    export interface SampledAudioSink {
        feedSample(value: number, count: number): void;
    }
    export interface SampledAudioSource {
        getAudioParams(): SampledAudioParams;
        connectAudio(audio: SampledAudioSink): void;
    }
    export interface AcceptsROM {
        loadROM(data: Uint8Array, title?: string): void;
    }
    export interface Resettable {
        reset(): void;
    }
    export interface MemoryBusConnected {
        connectMemoryBus(bus: Bus): void;
    }
    export interface IOBusConnected {
        connectIOBus(bus: Bus): void;
    }
    export interface CPU extends MemoryBusConnected, Resettable, SavesState<any> {
        getPC(): number;
        getSP(): number;
        isStable(): boolean;
    }
    export interface HasCPU extends Resettable {
        cpu: CPU;
    }
    export interface Interruptable<IT> {
        interrupt(type: IT): void;
    }
    export interface SavesInputState<CS> {
        loadControlsState(cs: CS): void;
        saveControlsState(): CS;
    }
    export interface AcceptsKeyInput {
        setKeyInput(key: number, code: number, flags: number): void;
    }
    export interface AcceptsPaddleInput {
        setPaddleInput(controller: number, value: number): void;
    }
    export interface Probeable {
        connectProbe(probe: ProbeAll): void;
    }
    export function xorshift32(x: number): number;
    export interface ProbeTime {
        logClocks(clocks: number): any;
        logNewScanline(): any;
        logNewFrame(): any;
    }
    export interface ProbeCPU {
        logExecute(address: number, SP: number): any;
        logInterrupt(type: number): any;
        logIllegal(address: number): any;
    }
    export interface ProbeBus {
        logRead(address: number, value: number): any;
        logWrite(address: number, value: number): any;
    }
    export interface ProbeIO {
        logIORead(address: number, value: number): any;
        logIOWrite(address: number, value: number): any;
    }
    export interface ProbeVRAM {
        logVRAMRead(address: number, value: number): any;
        logVRAMWrite(address: number, value: number): any;
    }
    export interface ProbeAll extends ProbeTime, ProbeCPU, ProbeBus, ProbeIO, ProbeVRAM {
    }
    export class NullProbe implements ProbeAll {
        logClocks(): void;
        logNewScanline(): void;
        logNewFrame(): void;
        logExecute(): void;
        logInterrupt(): void;
        logRead(): void;
        logWrite(): void;
        logIORead(): void;
        logIOWrite(): void;
        logVRAMRead(): void;
        logVRAMWrite(): void;
        logIllegal(): void;
    }
    export interface BasicMachineControlsState {
        inputs: Uint8Array;
    }
    export interface BasicMachineState extends BasicMachineControlsState {
        c: any;
        ram: Uint8Array;
    }
    export abstract class BasicHeadlessMachine implements HasCPU, Bus, AcceptsROM, Probeable, SavesState<BasicMachineState>, SavesInputState<BasicMachineControlsState> {
        abstract cpuFrequency: number;
        abstract defaultROMSize: number;
        abstract cpu: CPU;
        abstract ram: Uint8Array;
        rom: Uint8Array;
        inputs: Uint8Array;
        handler: (key: any, code: any, flags: any) => void;
        nullProbe: NullProbe;
        probe: ProbeAll;
        abstract read(a: number): number;
        abstract write(a: number, v: number): void;
        setKeyInput(key: number, code: number, flags: number): void;
        connectProbe(probe: ProbeAll): void;
        reset(): void;
        loadROM(data: Uint8Array, title?: string): void;
        loadState(state: any): void;
        saveState(): {
            c: any;
            ram: Uint8Array;
            inputs: Uint8Array;
        };
        loadControlsState(state: any): void;
        saveControlsState(): {
            inputs: Uint8Array;
        };
        advanceCPU(): number;
        probeMemoryBus(membus: Bus): Bus;
        connectCPUMemoryBus(membus: Bus): void;
        probeIOBus(iobus: Bus): Bus;
        connectCPUIOBus(iobus: Bus): void;
    }
    export abstract class BasicMachine extends BasicHeadlessMachine implements SampledAudioSource {
        abstract canvasWidth: number;
        abstract numVisibleScanlines: number;
        abstract sampleRate: number;
        overscan: boolean;
        rotate: number;
        pixels: Uint32Array;
        audio: SampledAudioSink;
        scanline: number;
        getAudioParams(): SampledAudioParams;
        connectAudio(audio: SampledAudioSink): void;
        getVideoParams(): VideoParams;
        connectVideo(pixels: Uint32Array): void;
    }
    export abstract class BasicScanlineMachine extends BasicMachine implements RasterFrameBased {
        abstract numTotalScanlines: number;
        abstract cpuCyclesPerLine: number;
        abstract startScanline(): void;
        abstract drawScanline(): void;
        frameCycles: number;
        advanceFrame(trap: TrapCondition): number;
        preFrame(): void;
        postFrame(): void;
        getRasterY(): number;
        getRasterX(): number;
    }
}
declare module "cpu/ZilogZ80" {
    import { CPU, Bus, InstructionBased, IOBusConnected, SavesState, Interruptable } from "devices";
    export interface Z80State {
        AF: number;
        BC: number;
        DE: number;
        HL: number;
        AF_: number;
        BC_: number;
        DE_: number;
        HL_: number;
        IX: number;
        IY: number;
        SP: number;
        PC: number;
        IR: number;
        iff1: number;
        iff2: number;
        im: number;
        halted: boolean;
        do_delayed_di: boolean;
        do_delayed_ei: boolean;
        cycle_counter: number;
    }
    export class Z80 implements CPU, InstructionBased, IOBusConnected, SavesState<Z80State>, Interruptable<number> {
        cpu: any;
        interruptType: any;
        memBus: Bus;
        ioBus: Bus;
        retryInterrupts: boolean;
        retryData: number;
        private buildCPU;
        connectMemoryBus(bus: Bus): void;
        connectIOBus(bus: Bus): void;
        advanceInsn(): any;
        reset(): void;
        interrupt(data: number): void;
        NMI(): void;
        getSP(): any;
        getPC(): any;
        isHalted(): any;
        saveState(): any;
        loadState(s: any): void;
        isStable(): boolean;
    }
}
declare module "audio" {
    export class MasterAudio {
        master: any;
        looper: any;
        start(): void;
        stop(): void;
    }
    export class AY38910_Audio {
        master: MasterAudio;
        psg: any;
        curreg: number;
        constructor(master: MasterAudio);
        reset(): void;
        selectRegister(val: number): void;
        setData(val: number): void;
        readData(): any;
        currentRegister(): number;
    }
    export class SN76489_Audio {
        master: MasterAudio;
        psg: any;
        constructor(master: MasterAudio);
        reset(): void;
        setData(val: number): void;
    }
    export function newPOKEYAudio(count: number): MasterAudio;
    export var POKEYDeviceChannel: () => void;
    export var WorkerSoundChannel: (worker: any) => void;
    export var SampleAudio: (clockfreq: any) => void;
    export class SampledAudio {
        sa: any;
        constructor(sampleRate: number);
        feedSample(value: number, count: number): void;
        start(): void;
        stop(): void;
    }
    import { SampledAudioSink } from "devices";
    export class TssChannelAdapter {
        channel: any;
        audioGain: number;
        constructor(channel: any, oversample: number, sampleRate: number);
        generate(sink: SampledAudioSink): void;
    }
}
declare module "recorder" {
    import { Platform, EmuState, EmuControlsState, EmuRecorder } from "baseplatform";
    type FrameRec = {
        controls: EmuControlsState;
        seed: number;
    };
    export class StateRecorderImpl implements EmuRecorder {
        checkpointInterval: number;
        callbackStateChanged: () => void;
        callbackNewCheckpoint: (state: EmuState) => void;
        maxCheckpoints: number;
        platform: Platform;
        checkpoints: EmuState[];
        framerecs: FrameRec[];
        frameCount: number;
        lastSeekFrame: number;
        constructor(platform: Platform);
        reset(): void;
        frameRequested(): boolean;
        numFrames(): number;
        currentFrame(): number;
        recordFrame(state: EmuState): void;
        getStateAtOrBefore(frame: number): {
            frame: number;
            state: EmuState;
        };
        loadFrame(seekframe: number): number;
        loadControls(frame: number): void;
        getLastCheckpoint(): EmuState;
    }
    import { Probeable, ProbeAll } from "devices";
    export enum ProbeFlags {
        CLOCKS = 0,
        EXECUTE = 16777216,
        MEM_READ = 33554432,
        MEM_WRITE = 50331648,
        IO_READ = 67108864,
        IO_WRITE = 83886080,
        VRAM_READ = 100663296,
        VRAM_WRITE = 117440512,
        INTERRUPT = 134217728,
        ILLEGAL = 150994944,
        SP_PUSH = 167772160,
        SP_POP = 184549376,
        SCANLINE = 2113929216,
        FRAME = 2130706432
    }
    export class ProbeRecorder implements ProbeAll {
        buf: Uint32Array;
        idx: number;
        fclk: number;
        sl: number;
        cur_sp: number;
        m: Probeable;
        singleFrame: boolean;
        constructor(m: Probeable);
        start(): void;
        stop(): void;
        reset(): void;
        log(a: number): void;
        relog(a: number): void;
        lastOp(): number;
        lastAddr(): number;
        logClocks(clocks: number): void;
        logNewScanline(): void;
        logNewFrame(): void;
        logExecute(address: number, SP: number): void;
        logInterrupt(type: number): void;
        logRead(address: number, value: number): void;
        logWrite(address: number, value: number): void;
        logIORead(address: number, value: number): void;
        logIOWrite(address: number, value: number): void;
        logVRAMRead(address: number, value: number): void;
        logVRAMWrite(address: number, value: number): void;
        logIllegal(address: number): void;
    }
}
declare module "baseplatform" {
    import { RasterVideo, AnimationTimer, ControllerPoller } from "emu";
    import { CodeAnalyzer } from "analysis";
    import { Segment } from "workertypes";
    export interface OpcodeMetadata {
        minCycles: number;
        maxCycles: number;
        insnlength: number;
        opcode: number;
    }
    export interface CpuState {
        PC: number;
        EPC?: number;
        o?: number;
        SP?: number;
    }
    export interface EmuState {
        c?: CpuState;
        b?: Uint8Array | number[];
        ram?: Uint8Array;
        o?: {};
    }
    export interface EmuControlsState {
    }
    export type DisasmLine = {
        line: string;
        nbytes: number;
        isaddr: boolean;
    };
    export type SymbolMap = {
        [ident: string]: number;
    };
    export type AddrSymbolMap = {
        [address: number]: string;
    };
    export class DebugSymbols {
        symbolmap: SymbolMap;
        addr2symbol: AddrSymbolMap;
        constructor(symbolmap: SymbolMap);
    }
    type MemoryMap = {
        [type: string]: Segment[];
    };
    export function isDebuggable(arg: any): arg is Debuggable;
    export interface Debuggable {
        getDebugCategories?(): string[];
        getDebugInfo?(category: string, state: EmuState): string;
    }
    export interface Platform {
        start(): void | Promise<void>;
        reset(): void;
        isRunning(): boolean;
        getToolForFilename(s: string): string;
        getDefaultExtension(): string;
        getPresets(): Preset[];
        pause(): void;
        resume(): void;
        loadROM(title: string, rom: any): any;
        loadBIOS?(title: string, rom: Uint8Array): any;
        loadState?(state: EmuState): void;
        saveState?(): EmuState;
        loadControlsState?(state: EmuControlsState): void;
        saveControlsState?(): EmuControlsState;
        inspect?(ident: string): string;
        disassemble?(addr: number, readfn: (addr: number) => number): DisasmLine;
        readAddress?(addr: number): number;
        readVRAMAddress?(addr: number): number;
        setFrameRate?(fps: number): void;
        getFrameRate?(): number;
        setupDebug?(callback: BreakpointCallback): void;
        clearDebug?(): void;
        step?(): void;
        runToVsync?(): void;
        runToPC?(pc: number): void;
        runUntilReturn?(): void;
        stepBack?(): void;
        runEval?(evalfunc: DebugEvalCondition): void;
        runToFrameClock?(clock: number): void;
        getOpcodeMetadata?(opcode: number, offset: number): OpcodeMetadata;
        getSP?(): number;
        getPC?(): number;
        getOriginPC?(): number;
        newCodeAnalyzer?(): CodeAnalyzer;
        getPlatformName?(): string;
        getMemoryMap?(): MemoryMap;
        setRecorder?(recorder: EmuRecorder): void;
        advance?(novideo?: boolean): void;
        showHelp?(tool: string, ident?: string): void;
        resize?(): void;
        getRasterScanline?(): number;
        setBreakpoint?(id: string, cond: DebugCondition): any;
        clearBreakpoint?(id: string): any;
        getCPUState?(): CpuState;
        debugSymbols?: DebugSymbols;
        startProbing?(): ProbeRecorder;
        stopProbing?(): void;
    }
    export interface Preset {
        id: string;
        name: string;
        chapter?: number;
        title?: string;
    }
    export interface MemoryBus {
        read: (address: number) => number;
        write: (address: number, value: number) => void;
        contend?: (address: number, cycles: number) => number;
        isContended?: (address: number) => boolean;
    }
    export type DebugCondition = () => boolean;
    export type DebugEvalCondition = (c: CpuState) => boolean;
    export type BreakpointCallback = (s: EmuState, msg?: string) => void;
    export class BreakpointList {
        id2bp: {
            [id: string]: Breakpoint;
        };
        getDebugCondition(): DebugCondition;
    }
    export type Breakpoint = {
        cond: DebugCondition;
    };
    export interface EmuRecorder {
        frameRequested(): boolean;
        recordFrame(state: EmuState): any;
    }
    export abstract class BasePlatform {
        recorder: EmuRecorder;
        debugSymbols: DebugSymbols;
        abstract loadState(state: EmuState): void;
        abstract saveState(): EmuState;
        abstract pause(): void;
        abstract resume(): void;
        abstract advance(novideo?: boolean): void;
        setRecorder(recorder: EmuRecorder): void;
        updateRecorder(): void;
    }
    export abstract class BaseDebugPlatform extends BasePlatform {
        onBreakpointHit: BreakpointCallback;
        debugCallback: DebugCondition;
        debugSavedState: EmuState;
        debugBreakState: EmuState;
        debugTargetClock: number;
        debugClock: number;
        breakpoints: BreakpointList;
        frameCount: number;
        abstract getCPUState(): CpuState;
        abstract readAddress(addr: number): number;
        setBreakpoint(id: string, cond: DebugCondition): void;
        clearBreakpoint(id: string): void;
        getDebugCallback(): DebugCondition;
        setupDebug(callback: BreakpointCallback): void;
        clearDebug(): void;
        setDebugCondition(debugCond: DebugCondition): void;
        restartDebugging(): void;
        preFrame(): void;
        postFrame(): void;
        pollControls(): void;
        nextFrame(novideo: boolean): void;
        abstract getSP(): number;
        abstract getPC(): number;
        abstract isStable(): boolean;
        evalDebugCondition(): void;
        wasBreakpointHit(): boolean;
        breakpointHit(targetClock: number, reason?: string): void;
        haltAndCatchFire(reason: string): void;
        runEval(evalfunc: DebugEvalCondition): void;
        runUntilReturn(): void;
        runToFrameClock(clock: number): void;
        step(): void;
        stepBack(): void;
        runToVsync(): void;
    }
    export function getToolForFilename_6502(fn: string): string;
    export abstract class Base6502Platform extends BaseDebugPlatform {
        debugPCDelta: number;
        fixPC(c: any): any;
        unfixPC(c: any): any;
        getSP(): number;
        getPC(): number;
        isStable(): boolean;
        newCPU(membus: MemoryBus): any;
        getOpcodeMetadata(opcode: any, offset: any): {
            opcode: any;
            minCycles: number;
            maxCycles: number;
            insnlength: number;
        };
        getOriginPC(): number;
        disassemble(pc: number, read: (addr: number) => number): DisasmLine;
        getToolForFilename: typeof getToolForFilename_6502;
        getDefaultExtension(): string;
        getDebugCategories(): string[];
        getDebugInfo(category: string, state: EmuState): string;
    }
    export function cpuStateToLongString_6502(c: any): string;
    export function getOpcodeMetadata_6502(opcode: any, address: any): {
        opcode: any;
        minCycles: number;
        maxCycles: number;
        insnlength: number;
    };
    export function cpuStateToLongString_Z80(c: any): string;
    export abstract class BaseZ80Platform extends BaseDebugPlatform {
        _cpu: any;
        waitCycles: number;
        newCPU(membus: MemoryBus, iobus: MemoryBus): any;
        getPC(): any;
        getSP(): any;
        isStable(): boolean;
        runCPU(cpu: any, cycles: number): number;
        getToolForFilename: typeof getToolForFilename_z80;
        getDefaultExtension(): string;
        getDebugCategories(): string[];
        getDebugInfo(category: string, state: EmuState): string;
        disassemble(pc: number, read: (addr: number) => number): DisasmLine;
    }
    export function getToolForFilename_z80(fn: any): string;
    export function cpuStateToLongString_6809(c: any): string;
    export function getToolForFilename_6809(fn: string): string;
    export abstract class Base6809Platform extends BaseZ80Platform {
        newCPU(membus: MemoryBus): any;
        getPC(): any;
        getSP(): any;
        cpuStateToLongString(c: CpuState): string;
        disassemble(pc: number, read: (addr: number) => number): DisasmLine;
        getDefaultExtension(): string;
        getToolForFilename: typeof getToolForFilename_6809;
        getDebugCategories(): string[];
        getDebugInfo(category: string, state: EmuState): string;
    }
    export abstract class BaseMAMEPlatform {
        loaded: boolean;
        preinitted: boolean;
        started: boolean;
        romfn: string;
        romdata: Uint8Array;
        video: any;
        running: boolean;
        console_vars: {
            [varname: string]: string[];
        };
        console_varname: string;
        initluavars: boolean;
        luadebugscript: string;
        js_lua_string: any;
        onBreakpointHit: any;
        mainElement: HTMLElement;
        constructor(mainElement: any);
        luareset(): void;
        luacall(s: any): void;
        pause(): void;
        resume(): void;
        reset(): void;
        isRunning(): boolean;
        bufferConsoleOutput(s: any): void;
        startModule(mainElement: any, opts: any): void;
        loadROMFile(data: any): void;
        loadRegion(region: any, data: any): void;
        preserveState(): {
            c: {};
        };
        initlua(): void;
    }
    export function dumpStackToString(platform: Platform, mem: Uint8Array | number[], start: number, end: number, sp: number, jsrop: number): string;
    export function lookupSymbol(platform: Platform, addr: number, extra: boolean): string;
    import { Bus, Resettable, FrameBased, AcceptsROM, SavesState, SavesInputState, HasCPU, TrapCondition, CPU } from "devices";
    import { SampledAudioSink } from "devices";
    import { SampledAudio } from "audio";
    import { ProbeRecorder } from "recorder";
    interface Machine extends Bus, Resettable, FrameBased, AcceptsROM, HasCPU, SavesState<EmuState>, SavesInputState<any> {
    }
    export abstract class BaseMachinePlatform<T extends Machine> extends BaseDebugPlatform implements Platform {
        machine: T;
        mainElement: HTMLElement;
        timer: AnimationTimer;
        video: RasterVideo;
        audio: SampledAudio;
        poller: ControllerPoller;
        probeRecorder: ProbeRecorder;
        startProbing: any;
        stopProbing: any;
        abstract newMachine(): T;
        abstract getToolForFilename(s: string): string;
        abstract getDefaultExtension(): string;
        abstract getPresets(): Preset[];
        constructor(mainElement: HTMLElement);
        reset(): void;
        loadState(s: any): void;
        saveState(): EmuState;
        getSP(): number;
        getPC(): number;
        isStable(): boolean;
        getCPUState(): any;
        loadControlsState(s: any): void;
        saveControlsState(): any;
        start(): void;
        loadROM(title: any, data: any): void;
        pollControls(): void;
        advance(novideo: boolean): void;
        isRunning(): boolean;
        resume(): void;
        pause(): void;
        getRasterScanline(): number;
    }
    export abstract class Base6502MachinePlatform<T extends Machine> extends BaseMachinePlatform<T> {
        getOpcodeMetadata: typeof getOpcodeMetadata_6502;
        getToolForFilename: typeof getToolForFilename_6502;
        disassemble(pc: number, read: (addr: number) => number): DisasmLine;
        getDebugCategories(): string[];
        getDebugInfo(category: string, state: EmuState): string;
    }
    export abstract class BaseZ80MachinePlatform<T extends Machine> extends BaseMachinePlatform<T> {
        getToolForFilename: typeof getToolForFilename_z80;
        getDebugCategories(): string[];
        getDebugInfo(category: string, state: EmuState): string;
        disassemble(pc: number, read: (addr: number) => number): DisasmLine;
    }
    export class WASMMachine implements Machine {
        prefix: string;
        instance: WebAssembly.Instance;
        exports: any;
        sys: number;
        pixel_dest: Uint32Array;
        pixel_src: Uint32Array;
        romptr: number;
        romlen: number;
        romarr: Uint8Array;
        stateptr: number;
        statearr: Uint8Array;
        cpustateptr: number;
        cpustatearr: Uint8Array;
        cpu: CPU;
        audio: SampledAudioSink;
        audioarr: Float32Array;
        prgstart: number;
        initstring: string;
        initindex: number;
        joymask0: number;
        constructor(prefix: string);
        loadWASM(): Promise<void>;
        reset(): void;
        getPC(): number;
        getSP(): number;
        isStable(): boolean;
        loadROM(rom: Uint8Array): void;
        advanceFrame(trap: TrapCondition): number;
        typeInitString(): void;
        read(address: number): number;
        readConst(address: number): number;
        write(address: number, value: number): void;
        getCPUState(): {
            PC: number;
            SP: number;
            A: number;
            X: number;
            Y: number;
            C: number;
            Z: number;
            I: number;
            D: number;
            V: number;
            N: number;
            o: number;
        };
        saveState(): {
            c: {
                PC: number;
                SP: number;
                A: number;
                X: number;
                Y: number;
                C: number;
                Z: number;
                I: number;
                D: number;
                V: number;
                N: number;
                o: number;
            };
            state: Uint8Array;
            ram: Uint8Array;
        };
        loadState(state: any): void;
        saveControlsState(): any;
        loadControlsState(state: any): void;
        getVideoParams(): {
            width: number;
            height: number;
            overscan: boolean;
            videoFrequency: number;
        };
        getAudioParams(): {
            sampleRate: number;
            stereo: boolean;
        };
        connectVideo(pixels: Uint32Array): void;
        syncVideo(): void;
        setKeyInput(key: number, code: number, flags: number): void;
        connectAudio(audio: SampledAudioSink): void;
        syncAudio(): void;
    }
}
declare module "analysis" {
    import { Platform } from "baseplatform";
    export interface CodeAnalyzer {
        showLoopTimingForPC(pc: number): any;
        pc2minclocks: {
            [key: number]: number;
        };
        pc2maxclocks: {
            [key: number]: number;
        };
        MAX_CLOCKS: number;
    }
    abstract class CodeAnalyzer6502 implements CodeAnalyzer {
        pc2minclocks: {};
        pc2maxclocks: {};
        START_CLOCKS: number;
        MAX_CLOCKS: number;
        WRAP_CLOCKS: boolean;
        jsrresult: {};
        platform: Platform;
        MAX_CYCLES: number;
        constructor(platform: Platform);
        getClockCountsAtPC(pc: any): import("baseplatform").OpcodeMetadata;
        traceInstructions(pc: number, minclocks: number, maxclocks: number, subaddr: number, constraints: any): void;
        showLoopTimingForPC(pc: number): void;
    }
    export class CodeAnalyzer_vcs extends CodeAnalyzer6502 {
        constructor(platform: Platform);
    }
    export class CodeAnalyzer_nes extends CodeAnalyzer6502 {
        constructor(platform: Platform);
    }
}
declare module "cpu/MOS6502" {
    import { CPU, Bus, ClockBased, SavesState, Interruptable } from "devices";
    export interface MOS6502State {
        PC: number;
        SP: number;
        A: number;
        X: number;
        Y: number;
        N: number;
        V: number;
        D: number;
        I: number;
        Z: number;
        C: number;
        T: number;
        o: number;
        R: number;
        d: number;
        AD: number;
        BA: number;
        BC: number;
        IA: number;
        bo: number;
        boa: number;
    }
    export enum MOS6502Interrupts {
        None = 0,
        NMI = 1,
        IRQ = 2
    }
    export class MOS6502 implements CPU, ClockBased, SavesState<MOS6502State>, Interruptable<MOS6502Interrupts> {
        cpu: any;
        interruptType: MOS6502Interrupts;
        connectMemoryBus(bus: Bus): void;
        advanceClock(): void;
        advanceInsn(): void;
        reset(): void;
        interrupt(itype: number): void;
        NMI(): void;
        IRQ(): void;
        getSP(): any;
        getPC(): any;
        saveState(): any;
        loadState(s: any): void;
        isStable(): boolean;
    }
}
declare module "video/tms9918a" {
    import { ProbeVRAM } from "devices";
    interface CPUInterface {
        setVDPInterrupt(b: boolean): any;
    }
    export class TMS9918A {
        cru: {
            setVDPInterrupt: (b: boolean) => void;
        };
        enableFlicker: boolean;
        probe: ProbeVRAM;
        ram: Uint8Array;
        registers: Uint8Array;
        spriteBuffer: Uint8Array;
        addressRegister: number;
        statusRegister: number;
        palette: number[];
        latch: boolean;
        prefetchByte: number;
        displayOn: boolean;
        interruptsOn: boolean;
        screenMode: number;
        bitmapMode: boolean;
        textMode: boolean;
        colorTable: number;
        nameTable: number;
        charPatternTable: number;
        spriteAttributeTable: number;
        spritePatternTable: number;
        colorTableMask: number;
        patternTableMask: number;
        ramMask: number;
        fgColor: number;
        bgColor: number;
        flicker: boolean;
        redrawRequired: boolean;
        fb32: Uint32Array;
        width: number;
        height: number;
        constructor(fb32: Uint32Array, cru: CPUInterface, enableFlicker: boolean);
        reset(): void;
        drawScanline(y: number): void;
        setReadAddress(i: number): void;
        setWriteAddress(i: number): void;
        setVDPWriteRegister(i: number): void;
        setVDPWriteCommand3(i: number): void;
        writeAddress(i: number): void;
        updateMode(reg0: number, reg1: number): void;
        updateTableMasks(): void;
        writeData(i: number): void;
        readStatus(): number;
        readData(): number;
        getRAM(): Uint8Array;
        colorTableSize(): number;
        patternTableSize(): number;
        getDebugTables(): [string, number, number][];
        getRegsString(): string;
        hexView(start: number, length: number, anchorAddr: number): {
            text: any;
            lineCount: any;
            anchorLine: any;
        };
        getWord(addr: number): number;
        getCharAt(x: number, y: number): number;
        setFlicker(value: boolean): void;
        getState(): {
            ram: Uint8Array;
            registers: Uint8Array;
            addressRegister: number;
            statusRegister: number;
            latch: boolean;
            prefetchByte: number;
            displayOn: boolean;
            interruptsOn: boolean;
            screenMode: number;
            bitmapMode: boolean;
            textMode: boolean;
            colorTable: number;
            nameTable: number;
            charPatternTable: number;
            spriteAttributeTable: number;
            spritePatternTable: number;
            colorTableMask: number;
            patternTableMask: number;
            ramMask: number;
            fgColor: number;
            bgColor: number;
            flicker: boolean;
        };
        restoreState(state: any): void;
    }
    export class SMSVDP extends TMS9918A {
        writeToCRAM: boolean;
        cram: Uint8Array;
        cpalette: Uint32Array;
        registers: Uint8Array;
        vramUntwiddled: Uint8Array;
        numVisibleLines: number;
        lineCounter: number;
        lineInterruptPending: boolean;
        reset(): void;
        readStatus(): number;
        updateMode(reg0: number, reg1: number): void;
        setReadAddress(i: number): void;
        setWriteAddress(i: number): void;
        setVDPWriteRegister(i: number): void;
        setVDPWriteCommand3(i: number): void;
        writeData(i: number): void;
        writeTwiddled(vdp_addr: number, val: number): void;
        getState(): {
            ram: Uint8Array;
            registers: Uint8Array;
            addressRegister: number;
            statusRegister: number;
            latch: boolean;
            prefetchByte: number;
            displayOn: boolean;
            interruptsOn: boolean;
            screenMode: number;
            bitmapMode: boolean;
            textMode: boolean;
            colorTable: number;
            nameTable: number;
            charPatternTable: number;
            spriteAttributeTable: number;
            spritePatternTable: number;
            colorTableMask: number;
            patternTableMask: number;
            ramMask: number;
            fgColor: number;
            bgColor: number;
            flicker: boolean;
        };
        restoreState(state: any): void;
        drawScanline(y: number): void;
        findSprites(line: number): any[];
        rasterize_background(lineAddr: number, pixelOffset: number, tileData: number, tileDef: number, transparent: boolean): void;
        clear_background(lineAddr: number, pixelOffset: number): void;
        rasterize_background_line(lineAddr: number, pixelOffset: number, nameAddr: number, yMod: number): void;
        rasterize_foreground_line(lineAddr: number, pixelOffset: number, nameAddr: number, yMod: number): void;
        rasterize_sprites(line: number, lineAddr: number, pixelOffset: number, sprites: any): void;
        border_clear(lineAddr: number, count: number): void;
        rasterize_line(line: number): void;
        getDebugTables(): [string, number, number][];
    }
}
